@@ -72,7 +72,8 @@ def issues_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
         if ported not in ('backported', 'major', ''):
             match = release_line_re.match(ported)
             if not match:
-                raise ValueError("Gave unknown issue metadata '%s' for issue no. %s" % (ported, issue_no))
+                err = "Gave unknown issue metadata '{0} for issue no. {1}"
+                raise ValueError(err.format(ported, issue_no))
             else:
                 line = match.groups()[0]
         # Create temporary node w/ data & final nodes to publish
@@ -105,7 +106,8 @@ def release_nodes(text, slug, date, config):
     datespan = ''
     if date:
         datespan = ' <span style="font-size: 75%%;">{0}</span>'.format(date)
-    header = '<h2 style="margin-bottom: 0.3em;">{0}{1}</h2>'.format(link, datespan)
+    header = '<h2 style="margin-bottom: 0.3em;">{0}{1}</h2>'.format(
+        link, datespan)
     return nodes.section('',
         nodes.raw(rawtext='', text=header, format='html'),
         ids=[text]
@@ -161,6 +163,16 @@ def append_unreleased_entries(app, lines, releases):
         })
 
 
+def reorder_release_entries(releases):
+    """
+    Mutate ``releases`` so the entrylist in each is ordered by feature/bug/etc.
+    """
+    order = {'feature': 0, 'bug': 1, 'support': 2}
+    for release in releases:
+        entries = release['entries'][:]
+        release['entries'] = sorted(entries, key=lambda x: order[x.type])
+
+
 def construct_entry_with_release(focus, issues, lines, log, releases, rest):
     """
     Releases 'eat' the entries in their line's list and get added to the
@@ -346,7 +358,9 @@ def construct_releases(entries, app):
         # Release lines, once the release obj is removed, should be empty or a
         # comma-separated list of issue numbers.
         if isinstance(focus, Release):
-            construct_entry_with_release(focus, issues, lines, log, releases, rest)
+            construct_entry_with_release(
+                focus, issues, lines, log, releases, rest
+            )
 
         # Entries get copied into release line buckets as follows:
         # * Features and support go into 'unreleased_feature' for use in new
@@ -368,6 +382,8 @@ def construct_releases(entries, app):
 
     append_unreleased_entries(app, lines, releases)
 
+    reorder_release_entries(releases)
+
     return releases
 
 
@@ -414,17 +430,36 @@ def construct_nodes(releases):
     return result
 
 
+class BulletListVisitor(nodes.NodeVisitor):
+    def __init__(self, document, app):
+        nodes.NodeVisitor.__init__(self, document)
+        self.found_changelog = False
+        self.app = app
+
+    def visit_bullet_list(self, node):
+        # The first found bullet list (which should be the first one at the top
+        # level of the document) is the changelog.
+        if not self.found_changelog:
+            self.found_changelog = True
+            # Walk + parse into release mapping
+            releases = construct_releases(node.children, self.app)
+            # Construct new set of nodes to replace the old, and we're done
+            node.replace_self(construct_nodes(releases))
+
+    def unknown_visit(self, node):
+        pass
+
+
 def generate_changelog(app, doctree):
+    # Don't scan/mutate documents that don't match the configured document name
+    # (which by default is changelog.rst).
     if app.env.docname != app.config.releases_document_name:
         return
-    # Second item inside main document is the 'modern' changelog bullet-list
-    # object, whose children are the nodes we care about.
-    source = doctree[0]
-    changelog = source.children.pop(1)
-    # Walk + parse into release mapping
-    releases = construct_releases(changelog.children, app)
-    # Construct new set of nodes to replace the old, and we're done
-    source[1:1] = construct_nodes(releases)
+
+    # Find the first bullet-list node & replace it with our organized/parsed
+    # elements.
+    changelog_visitor = BulletListVisitor(doctree, app)
+    doctree.walk(changelog_visitor)
 
 
 def setup(app):
