@@ -2,13 +2,13 @@ from tempfile import mkdtemp
 from shutil import rmtree
 
 import six
-from spec import Spec, skip, eq_, raises
+from spec import Spec, eq_, raises
 from mock import Mock
 from docutils.nodes import (
-    reference, bullet_list, list_item, title, raw, paragraph, Text, section,
+    reference, bullet_list, list_item, raw, paragraph, Text,
 )
-from docutils.utils import new_document
 from sphinx.application import Sphinx
+import sphinx
 
 from releases import (
     Issue,
@@ -17,7 +17,6 @@ from releases import (
     release_role,
     construct_releases,
     construct_nodes,
-    generate_changelog,
 )
 from releases import setup as releases_setup # avoid unittest crap
 
@@ -26,14 +25,14 @@ def _app(**kwargs):
     # Create a real Sphinx app, with stupid temp dirs because it assumes.
     # Helps catch things like "testing a config option but forgot
     # app.add_config_value()"
-    src, doctree = mkdtemp(), mkdtemp()
+    src, dst, doctree = mkdtemp(), mkdtemp(), mkdtemp()
     try:
         # STFU Sphinx :(
         Sphinx._log = lambda self, message, wfile, nonl=False: None
         app = Sphinx(
             srcdir=src,
             confdir=None,
-            outdir=None,
+            outdir=dst,
             doctreedir=doctree,
             buildername='html',
         )
@@ -54,7 +53,11 @@ def _app(**kwargs):
         config['releases_{0}'.format(name)] = kwargs[name]
     # Stitch together as the sphinx app init() usually does w/ real conf files
     app.config._raw_config = config
-    app.config.init_values()
+    # init_values() requires a 'warn' runner on Sphinx 1.3+, give it no-op.
+    init_args = []
+    if sphinx.version_info[:2] > (1, 2):
+        init_args = [lambda x: x]
+    app.config.init_values(*init_args)
     return app
 
 def _inliner(app=None):
@@ -329,6 +332,22 @@ class releases(Spec):
         cl = _changelog2dict(cl)
         assert len(cl['1.0.1']) == 2
 
+    def issues_are_sorted_by_type_within_releases(self):
+        b1 = _issue('bug', '123', major=True)
+        b2 = _issue('bug', '124', major=True)
+        s1 = _issue('support', '25')
+        s2 = _issue('support', '26')
+        f1 = _issue('feature', '3455')
+        f2 = _issue('feature', '3456')
+
+        # Semi random definitely-not-in-desired-order order
+        changelog = _changelog2dict(_releases('1.1', b1, s1, s2, f1, b2, f2))
+
+        # Order should be feature, bug, support. While it doesn't REALLY
+        # matter, assert that within each category the order matches the old
+        # 'reverse chronological' order.
+        eq_(changelog['1.1'], [f2, f1, b2, b1, s2, s1])
+
 
 def _obj2name(obj):
     cls = obj if isinstance(obj, type) else obj.__class__
